searchNodes=[{"ref":"QueryBuilder.html","title":"QueryBuilder","type":"module","doc":""},{"ref":"QueryBuilder.html#from_list/2","title":"QueryBuilder.from_list/2","type":"function","doc":"Allows to pass a list of operations through a keyword list. Example: QueryBuilder.from_list(query, [ where: [name: &quot;John&quot;, city: &quot;Anytown&quot;], preload: [articles: :comments] ])"},{"ref":"QueryBuilder.html#join/3","title":"QueryBuilder.join/3","type":"function","doc":"A join query expression. Third argument type may be passed one of the possible values for Ecto.Query.join/5&#39;s qualifier argument. Example: QueryBuilder.join(query, :articles, :left)"},{"ref":"QueryBuilder.html#or_where/2","title":"QueryBuilder.or_where/2","type":"function","doc":"An OR where query expression. Example: QueryBuilder.or_where(query, firstname: &quot;John&quot;)"},{"ref":"QueryBuilder.html#or_where/3","title":"QueryBuilder.or_where/3","type":"function","doc":"An OR where query expression. Associations are passed in second argument; fields from these associations can then be referenced by writing the field name, followed by the &quot;@&quot; character and the association name, as an atom. For example: :name@users. Example: QueryBuilder.or_where(query, [role: :permissions], name@permissions: :write)"},{"ref":"QueryBuilder.html#order_by/2","title":"QueryBuilder.order_by/2","type":"function","doc":"An order by query expression. Example: QueryBuilder.order_by(query, lastname: :asc, firstname: :asc)"},{"ref":"QueryBuilder.html#order_by/3","title":"QueryBuilder.order_by/3","type":"function","doc":"An order by query expression. For more about the second argument, see where/3. Example: QueryBuilder.order_by(query, :articles, title@articles: :asc)"},{"ref":"QueryBuilder.html#preload/2","title":"QueryBuilder.preload/2","type":"function","doc":"Preloads the associations. Bindings are automatically set if joins have been made, or if it is preferable to join (i.e. one-to-one associations are preferable to include into the query result rather than emitting separate DB queries). Example: QueryBuilder.preload(query, [role: :permissions, articles: [:stars, comments: :user]])"},{"ref":"QueryBuilder.html#where/2","title":"QueryBuilder.where/2","type":"function","doc":"An AND where query expression. Example: QueryBuilder.where(query, firstname: &quot;John&quot;)"},{"ref":"QueryBuilder.html#where/3","title":"QueryBuilder.where/3","type":"function","doc":"An AND where query expression. Associations are passed in second argument; fields from these associations can then be referenced by writing the field name, followed by the &quot;@&quot; character and the association name, as an atom. For example: :name@users. Example: QueryBuilder.where(query, [role: :permissions], name@permissions: :write)"},{"ref":"readme.html","title":"Query Builder","type":"extras","doc":"Query Builder Query Builder allows you to build and compose Ecto queries based on data. User |&gt; QueryBuilder.where(firstname: &quot;John&quot;) |&gt; QueryBuilder.where([{:age, :gt, 30}, city: &quot;Anytown&quot;]) |&gt; QueryBuilder.order_by(lastname: :asc) |&gt; QueryBuilder.preload([:role, authored_articles: :comments]) |&gt; Repo.all() This allows writing queries more concisely, without having to deal with bindings and macros. Its primary goal is to allow Context functions to receive a set of filters and options: Blog.list_articles(preload: :comments, order_by: [title: :asc]) Blog.list_articles(preload: [:category, comments: :user]) This avoids having to create many different functions in the Context for every combination of filters and options, or to create one general function that does too much to satisfy all the consumers. The calling code (e.g. the Controllers), can now retrieve the list of articles with different options. In some part of the application, the category is needed; in other parts it is not; sometimes the articles must be sorted based on their title; other times it doesn&#39;t matter, etc. See QueryBuilder.from_list/2 below."},{"ref":"readme.html#examples","title":"Query Builder - Examples","type":"extras","doc":"User |&gt; QueryBuilder.where(firstname: &quot;John&quot;) |&gt; QueryBuilder.where([{:age, :gt, 30}, city: &quot;Anytown&quot;]) |&gt; QueryBuilder.order_by(lastname: :asc) |&gt; QueryBuilder.preload([:role, authored_articles: :comments]) |&gt; Repo.all() Filtering on associations is supported: User |&gt; QueryBuilder.where(:role, name@role: &quot;admin&quot;) |&gt; Repo.all() User |&gt; QueryBuilder.where([role: :permissions], name@permissions: &quot;delete&quot;) |&gt; Repo.all() Article |&gt; QueryBuilder.where(:author, id@author: author_id) |&gt; QueryBuilder.where([:author, :comments], {:logged_at@author, :lt, :inserted_at@comments}) |&gt; QueryBuilder.preload(:comments) |&gt; Repo.all()"},{"ref":"readme.html#usage","title":"Query Builder - Usage","type":"extras","doc":"Add use QueryBuilder in your schema: defmodule MyApp.User do use Ecto.Schema use QueryBuilder schema &quot;users&quot; do # code end # code end You may also specify the schema&#39;s associations to QueryBuilder in order to remedy some limitations when building queries: defmodule MyApp.User do use Ecto.Schema use QueryBuilder, assoc_fields: [:role, :articles] schema &quot;users&quot; do # code belongs_to :role, MyApp.Role has_many :articles, MyApp.Article end # code end Currently, supported operations are: QueryBuilder.where/2 QueryBuilder.where(query, firstname: &quot;John&quot;) QueryBuilder.where/3 QueryBuilder.where(query, [role: :permissions], name@permissions: :write) QueryBuilder.or_where/2 QueryBuilder.or_where(query, firstname: &quot;John&quot;) QueryBuilder.or_where/3 QueryBuilder.or_where(query, :role, name@role: &quot;author&quot;) Above where/or_where functions support different filter operations, for instance: QueryBuilder.where(query, {:age, :greater_than, 18}) Supported filter operations are: :equal_to (or :eq) :other_than (or :ne) :greater_than (or :gt) :greater_than_or_equal_to (or :ge) :less_than (or :lt) :less_than_or_equal_to (or :le) :like :ilike :starts_with :ends_with :contains Array inclusion checking: :in :not_in :include :exclude Note that :starts_with, :ends_with and :contains operations can be written using :like, but offer a more declarative style and are safer, as they escape the % and _ characters for you. You may also perform case insensitive searchs using these functions. Example: QueryBuilder.where({:name, :starts_with, &quot;jo&quot;}) QueryBuilder.where({:name, :starts_with, &quot;jo&quot;, case: :insensitive}) # `:i` will also work When using :like or :ilike, make sure to escape % and _ characters properly. QueryBuilder.order_by/2 QueryBuilder.order_by(query, lastname: :asc, firstname: :asc) QueryBuilder.order_by/3 QueryBuilder.order_by(query, :articles, title@articles: :asc) QueryBuilder.preload/2 QueryBuilder.preload(query, [role: :permissions, articles: [:stars, comments: :user]]) QueryBuilder.join/3 QueryBuilder.join(query, :articles, :left) QueryBuilder.from_list/2 QueryBuilder.from_list(query, [ where: [name: &quot;John&quot;, city: &quot;Anytown&quot;], preload: [articles: :comments] ]) The QueryBuilder.from_list/2 function was the main motivation behind the writing of this library. As explained above, it allows to add querying options to the Context functions. Example: defmodule MyApp.Blog do alias MyApp.Blog.Article def get_article_by_id(id, opts \\\\ []) do QueryBuilder.where(Article, id: id) |&gt; QueryBuilder.from_list(opts) |&gt; Repo.one!() end end The function can now be called as follows (for instance, from a Controller): Blog.get_article_by_id(id, preload: [:comments]) Blog.get_article_by_id(id, preload: [:likes]) Blog.get_article_by_id( id, order_by: {:comments, inserted_at@comments: :desc}, preload: [comments: :user] )"},{"ref":"readme.html#installation","title":"Query Builder - Installation","type":"extras","doc":"Add query_builder for Elixir as a dependency in your mix.exs file: def deps do [ {:query_builder, &quot;~&gt; 0.13.0&quot;} ] end"},{"ref":"readme.html#hexdocs","title":"Query Builder - HexDocs","type":"extras","doc":"HexDocs documentation can be found at https://hexdocs.pm/query_builder."}]